#!/usr/bin/python
# -*- Mode: python; indent-tabs-mode: nil; tab-width: 4 -*- ## emacs weenies

# Copyright (C) 2012 Canonical, Ltd

# Authors:
#   First Last <first.last@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.



# usage: cp test_template.py test_yourthing.py ## and edit
# run: python path/to/test/test_template.py
# run all: nosetests --exe -d test

# Like code, tests must be committed and dput to work elsewhere.
# It is possible to build and test correctly locally, but fail
# remotely if you don't bzr push/dput the right stuff.


import sys
import os
import subprocess
import logging
from tempfile import mkdtemp
import shutil
import unittest

# Point at the objects under test.
sys.path.insert(0,
    os.path.abspath(os.path.dirname(os.path.abspath(__file__))
    + '/..'))
# Point at (possible) supporting test code.
sys.path.insert( 0,
    os.path.dirname(__file__ ) ) # in case you're running in place

me, _ = os.path.splitext(os.path.basename(__file__)) # name of this file, less .py

logging.basicConfig(level = logging.DEBUG) ## TODO: how can the test runner pass in args?

# Name must be <Something>Tests so test runner can find it
class TemplateExecutableTests(unittest.TestCase):
    """Describe the tests here"""

    # working_directory is a resource supporting the test
    # Create it before the tests are run. Remove it after the tests are run iff
    #   all tests succeed.
    working_directory = mkdtemp()

    @classmethod
    def setUpClass(cls):
        # Move to the working directory.
        os.chdir(cls.working_directory)
        logging.debug("current directory: %s" % cls.working_directory)

    @classmethod
    def tearDownClass(cls):
        # TODO: remove directory iff all tests pass
        shutil.rmtree(cls.working_directory)



    # Name must be test_<something> so test runner can find them.
    def test_always_succeeds(self):
        """Always succeeds. Tests the test runner."""
        self.assertTrue(True, "tests the test runner")

    @unittest.skip("here's how to skip a test with an annotation")
    def test_always_skips(self):
        """Example of a declarative skip"""
        self.fail("show how to skip a test declaratively")

    def test_demonstrates_runtime_skipping(self):
        """Example of a runtime skip"""
        self.skipTest("skipping at runtime")

    def test_that_runs_an_external_executable(self):
        command = ['ls', '-la', '/']
        p = subprocess.Popen(command, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        stdout, stderr = p.communicate()
        self.assertEquals(p.returncode, 0) # exit status 0
        self.assertEqual(stderr, '') # no errors reported to stderr
        self.assertGreater(len(stdout), 0) # should be some output





# This sequence lets you run the test suite with python directly rather
# than with a test runner indirectly.
def main():
    unittest.main()

if __name__ == '__main__':
    main()
